@@ -106,7 +106,7 @@ Overview
 """
 
 __author__ = "Hubert Pham"
-__version__ = "0.2.7"
+__version__ = "0.2.8"
 __docformat__ = "restructuredtext en"
 
 import sys
@@ -985,9 +985,21 @@ class PyAudio:
         :rtype: dict
         """
 
+        device_name = device_info.name
+
+        # Attempt to decode device_name
+        for codec in ["utf-8", "cp1252"]:
+            try:
+                device_name = device_name.decode(codec)
+                break
+            except:
+                pass
+
+        # If we fail to decode, we return the raw bytes and let the caller
+        # deal with the encoding.
         return {'index' : index,
                 'structVersion' : device_info.structVersion,
-                'name' : device_info.name,
+                'name' : device_name,
                 'hostApi' : device_info.hostApi,
                 'maxInputChannels' : device_info.maxInputChannels,
                 'maxOutputChannels' : device_info.maxOutputChannels,
